import java.applet.*;
import java.awt.*;
import java.util.Vector;

public class ScreenType extends Applet
{
    // key the keystrokes in a vector
    private Vector m_vKeys;

    // the current character location
    private Dimension m_dimLoc;

    // the font metrics for this applet's window
    private FontMetrics m_fm;

	public ScreenType()
	{
        // create a vector with enough room for 100 key
        // objects. Let it grow by 100 at a time. This will
        // be faster than the default.
        m_vKeys = new Vector(100, 100);

        // get a dimension object somewhere in the window
        m_dimLoc = new Dimension(10, 10);
	}

	public String getAppletInfo()
	{
		return "Name: ScreenType\r\n" +
		       "Author: Stephen R. Davis\r\n" +
		       "Created for Learn Java Now (c)";
	}


	public void init()
	{
		resize(320, 240);

        // get font metrics here to save time
        m_fm = getFontMetrics(getFont());
	}

	public void destroy()
	{
	}

	public void paint(Graphics g)
	{
        // loop thru the saved Key objects
        int nSize = m_vKeys.size();
        for (int i = 0; i < nSize; i++)
        {
            // get the key
            Key key = (Key)m_vKeys.elementAt(i);

            // from the key get the keystroke and the location
            Dimension dimLoc = key.GetLoc();
            char[] cKey = key.GetKey();

            // output the key and location
            g.drawChars(cKey, 0, 1, dimLoc.width, dimLoc.height);
        }
	}

	public void start()
	{
	}
	
	public void stop()
	{
	}


	public boolean mouseDown(Event evt, int x, int y)
	{
        // on double clicks...
        if (evt.clickCount > 1)
        {
            // ...clear everything out;...
            m_vKeys.removeAllElements();
            repaint();
        }
        else
        {
            // ...otherwise, just update the location
            m_dimLoc.width = x;
            m_dimLoc.height = y;
        }
		return true;
	}

	public boolean mouseUp(Event evt, int x, int y)
	{
		return true;
	}

    public boolean keyDown(Event evt, 
                           int nKey)
    {
        // add key at the current location
        m_vKeys.addElement(new Key(nKey, m_dimLoc));

        // perform a minimal repaint (just the box containing
        // the character)
        int nL, nR, nT, nB;
        nL = m_dimLoc.width;
        nR = nL + m_fm.charWidth(nKey);
        nB = m_dimLoc.height;
        nT = nB - m_fm.getHeight();

        repaint(nL, nT, nR, nB);

        // now update the location for the next character
        // by moving it over the width of this character
        // (remember that different characters are different
        // widths)
        m_dimLoc.width += m_fm.charWidth(nKey);

        return true;
    }
}



class Key
{
    private char[] m_cKey;
    private Dimension m_dimLoc;

    public Key(int nKey, Dimension dimLoc)
    {
        // store the character in an array of 1;
        // this is more convenient because drawChars()
        // back up in paint() wants an array
        m_cKey = new char[1];
        m_cKey[0] = (char)nKey;

        // use the same type constructor to make our own
        // copy of the Dimension object with the location
        m_dimLoc = new Dimension(dimLoc);
    }

    Dimension GetLoc()
    {
        return m_dimLoc;
    }

    char[] GetKey()
    {
        return m_cKey;
    }
}
